@@ -13,8 +13,10 @@
 # limitations under the License.
 
 """Functions for getting templates and calculating template features."""
+import abc
 import dataclasses
 import datetime
+import functools
 import glob
 import os
 import re
@@ -71,10 +73,6 @@ class DateError(PrefilterError):
   """An error indicating that the hit date was after the max allowed date."""
 
 
-class PdbIdError(PrefilterError):
-  """An error indicating that the hit PDB ID was identical to the query."""
-
-
 class AlignRatioError(PrefilterError):
   """An error indicating that the hit align ratio to the query was too small."""
 
@@ -128,7 +126,6 @@ def _is_after_cutoff(
   else:
     # Since this is just a quick prefilter to reduce the number of mmCIF files
     # we need to parse, we don't have to worry about returning True here.
-    logging.warning('Template structure not in release dates dict: %s', pdb_id)
     return False
 
 
@@ -177,7 +174,6 @@ def _assess_hhsearch_hit(
     hit: parsers.TemplateHit,
     hit_pdb_code: str,
     query_sequence: str,
-    query_pdb_code: Optional[str],
     release_dates: Mapping[str, datetime.datetime],
     release_date_cutoff: datetime.datetime,
     max_subsequence_ratio: float = 0.95,
@@ -190,7 +186,6 @@ def _assess_hhsearch_hit(
       different from the value in the actual hit since the original pdb might
       have become obsolete.
     query_sequence: Amino acid sequence of the query.
-    query_pdb_code: 4 letter pdb code of the query.
     release_dates: Dictionary mapping pdb codes to their structure release
       dates.
     release_date_cutoff: Max release date that is valid for this query.
@@ -202,7 +197,6 @@ def _assess_hhsearch_hit(
 
   Raises:
     DateError: If the hit date was after the max allowed date.
-    PdbIdError: If the hit PDB ID was identical to the query.
     AlignRatioError: If the hit align ratio to the query was too small.
     DuplicateError: If the hit was an exact subsequence of the query.
     LengthError: If the hit was too short.
@@ -222,10 +216,6 @@ def _assess_hhsearch_hit(
     raise DateError(f'Date ({release_dates[hit_pdb_code]}) > max template date '
                     f'({release_date_cutoff}).')
 
-  if query_pdb_code is not None:
-    if query_pdb_code.lower() == hit_pdb_code.lower():
-      raise PdbIdError('PDB code identical to Query PDB code.')
-
   if align_ratio <= min_align_ratio:
     raise AlignRatioError('Proportion of residues aligned to query too small. '
                           f'Align ratio: {align_ratio}.')
@@ -368,8 +358,9 @@ def _realign_pdb_template_to_query(
           'protein chain.')
 
   try:
-    (old_aligned_template, new_aligned_template), _ = parsers.parse_a3m(
+    parsed_a3m = parsers.parse_a3m(
         aligner.align([old_template_sequence, new_template_sequence]))
+    old_aligned_template, new_aligned_template = parsed_a3m.sequences
   except Exception as e:
     raise QueryToTemplateAlignError(
         'Could not align old template %s to template %s (%s_%s). Error: %s' %
@@ -472,6 +463,18 @@ def _get_atom_positions(
           pos[residue_constants.atom_order['SD']] = [x, y, z]
           mask[residue_constants.atom_order['SD']] = 1.0
 
+      # Fix naming errors in arginine residues where NH2 is incorrectly
+      # assigned to be closer to CD than NH1.
+      cd = residue_constants.atom_order['CD']
+      nh1 = residue_constants.atom_order['NH1']
+      nh2 = residue_constants.atom_order['NH2']
+      if (res.get_resname() == 'ARG' and
+          all(mask[atom_index] for atom_index in (cd, nh1, nh2)) and
+          (np.linalg.norm(pos[nh1] - pos[cd]) >
+           np.linalg.norm(pos[nh2] - pos[cd]))):
+        pos[nh1], pos[nh2] = pos[nh2].copy(), pos[nh1].copy()
+        mask[nh1], mask[nh2] = mask[nh2].copy(), mask[nh1].copy()
+
     all_positions[res_index] = pos
     all_positions_mask[res_index] = mask
   _check_residue_distances(
@@ -673,9 +676,15 @@ class SingleHitResult:
   warning: Optional[str]
 
 
+@functools.lru_cache(16, typed=False)
+def _read_file(path):
+  with open(path, 'r') as f:
+    file_data = f.read()
+  return file_data
+
+
 def _process_single_hit(
     query_sequence: str,
-    query_pdb_code: Optional[str],
     hit: parsers.TemplateHit,
     mmcif_dir: str,
     max_template_date: datetime.datetime,
@@ -702,14 +711,13 @@ def _process_single_hit(
         hit=hit,
         hit_pdb_code=hit_pdb_code,
         query_sequence=query_sequence,
-        query_pdb_code=query_pdb_code,
         release_dates=release_dates,
-        release_date_cutoff=max_template_date)
+        release_date_cutoff=max_template_date,
+        max_subsequence_ratio=1.0)
   except PrefilterError as e:
     msg = f'hit {hit_pdb_code}_{hit_chain_id} did not pass prefilter: {str(e)}'
-    logging.info('%s: %s', query_pdb_code, msg)
-    if strict_error_check and isinstance(
-        e, (DateError, PdbIdError, DuplicateError)):
+    logging.info(msg)
+    if strict_error_check and isinstance(e, (DateError, DuplicateError)):
       # In strict mode we treat some prefilter cases as errors.
       return SingleHitResult(features=None, error=msg, warning=None)
 
@@ -724,11 +732,10 @@ def _process_single_hit(
   template_sequence = hit.hit_sequence.replace('-', '')
 
   cif_path = os.path.join(mmcif_dir, hit_pdb_code + '.cif')
-  logging.info('Reading PDB entry from %s. Query: %s, template: %s',
-               cif_path, query_sequence, template_sequence)
+  logging.debug('Reading PDB entry from %s. Query: %s, template: %s', cif_path,
+                query_sequence, template_sequence)
   # Fail if we can't find the mmCIF file.
-  with open(cif_path, 'r') as cif_file:
-    cif_string = cif_file.read()
+  cif_string = _read_file(cif_path)
 
   parsing_result = mmcif_parsing.parse(
       file_id=hit_pdb_code, mmcif_string=cif_string)
@@ -742,7 +749,7 @@ def _process_single_hit(
       if strict_error_check:
         return SingleHitResult(features=None, error=error, warning=None)
       else:
-        logging.warning(error)
+        logging.debug(error)
         return SingleHitResult(features=None, error=None, warning=None)
 
   try:
@@ -754,7 +761,10 @@ def _process_single_hit(
         query_sequence=query_sequence,
         template_chain_id=hit_chain_id,
         kalign_binary_path=kalign_binary_path)
-    features['template_sum_probs'] = [hit.sum_probs]
+    if hit.sum_probs is None:
+      features['template_sum_probs'] = [0]
+    else:
+      features['template_sum_probs'] = [hit.sum_probs]
 
     # It is possible there were some errors when parsing the other chains in the
     # mmCIF file, but the template features for the chain we want were still
@@ -765,7 +775,7 @@ def _process_single_hit(
           TemplateAtomMaskAllZerosError) as e:
     # These 3 errors indicate missing mmCIF experimental data rather than a
     # problem with the template search, so turn them into warnings.
-    warning = ('%s_%s (sum_probs: %.2f, rank: %d): feature extracting errors: '
+    warning = ('%s_%s (sum_probs: %s, rank: %s): feature extracting errors: '
                '%s, mmCIF parsing errors: %s'
                % (hit_pdb_code, hit_chain_id, hit.sum_probs, hit.index,
                   str(e), parsing_result.errors))
@@ -788,8 +798,8 @@ class TemplateSearchResult:
   warnings: Sequence[str]
 
 
-class TemplateHitFeaturizer:
-  """A class for turning hhr hits to template features."""
+class TemplateHitFeaturizer(abc.ABC):
+  """An abstract base class for turning template hits to template features."""
 
   def __init__(
       self,
@@ -850,29 +860,28 @@ class TemplateHitFeaturizer:
     else:
       self._obsolete_pdbs = {}
 
+  @abc.abstractmethod
+  def get_templates(
+      self,
+      query_sequence: str,
+      hits: Sequence[parsers.TemplateHit]) -> TemplateSearchResult:
+    """Computes the templates for given query sequence."""
+
+
+class HhsearchHitFeaturizer(TemplateHitFeaturizer):
+  """A class for turning a3m hits from hhsearch to template features."""
+
   def get_templates(
       self,
       query_sequence: str,
-      query_pdb_code: Optional[str],
-      query_release_date: Optional[datetime.datetime],
       hits: Sequence[parsers.TemplateHit]) -> TemplateSearchResult:
     """Computes the templates for given query sequence (more details above)."""
-    logging.info('Searching for template for: %s', query_pdb_code)
+    logging.info('Searching for template for: %s', query_sequence)
 
     template_features = {}
     for template_feature_name in TEMPLATE_FEATURES:
       template_features[template_feature_name] = []
 
-    # Always use a max_template_date. Set to query_release_date minus 60 days
-    # if that's earlier.
-    template_cutoff_date = self._max_template_date
-    if query_release_date:
-      delta = datetime.timedelta(days=60)
-      if query_release_date - delta < template_cutoff_date:
-        template_cutoff_date = query_release_date - delta
-      assert template_cutoff_date < query_release_date
-    assert template_cutoff_date <= self._max_template_date
-
     num_hits = 0
     errors = []
     warnings = []
@@ -884,10 +893,9 @@ class TemplateHitFeaturizer:
 
       result = _process_single_hit(
           query_sequence=query_sequence,
-          query_pdb_code=query_pdb_code,
           hit=hit,
           mmcif_dir=self._mmcif_dir,
-          max_template_date=template_cutoff_date,
+          max_template_date=self._max_template_date,
           release_dates=self._release_dates,
           obsolete_pdbs=self._obsolete_pdbs,
           strict_error_check=self._strict_error_check,
@@ -920,3 +928,84 @@ class TemplateHitFeaturizer:
 
     return TemplateSearchResult(
         features=template_features, errors=errors, warnings=warnings)
+
+
+class HmmsearchHitFeaturizer(TemplateHitFeaturizer):
+  """A class for turning a3m hits from hmmsearch to template features."""
+
+  def get_templates(
+      self,
+      query_sequence: str,
+      hits: Sequence[parsers.TemplateHit]) -> TemplateSearchResult:
+    """Computes the templates for given query sequence (more details above)."""
+    logging.info('Searching for template for: %s', query_sequence)
+
+    template_features = {}
+    for template_feature_name in TEMPLATE_FEATURES:
+      template_features[template_feature_name] = []
+
+    already_seen = set()
+    errors = []
+    warnings = []
+
+    if not hits or hits[0].sum_probs is None:
+      sorted_hits = hits
+    else:
+      sorted_hits = sorted(hits, key=lambda x: x.sum_probs, reverse=True)
+
+    for hit in sorted_hits:
+      # We got all the templates we wanted, stop processing hits.
+      if len(already_seen) >= self._max_hits:
+        break
+
+      result = _process_single_hit(
+          query_sequence=query_sequence,
+          hit=hit,
+          mmcif_dir=self._mmcif_dir,
+          max_template_date=self._max_template_date,
+          release_dates=self._release_dates,
+          obsolete_pdbs=self._obsolete_pdbs,
+          strict_error_check=self._strict_error_check,
+          kalign_binary_path=self._kalign_binary_path)
+
+      if result.error:
+        errors.append(result.error)
+
+      # There could be an error even if there are some results, e.g. thrown by
+      # other unparsable chains in the same mmCIF file.
+      if result.warning:
+        warnings.append(result.warning)
+
+      if result.features is None:
+        logging.debug('Skipped invalid hit %s, error: %s, warning: %s',
+                      hit.name, result.error, result.warning)
+      else:
+        already_seen_key = result.features['template_sequence']
+        if already_seen_key in already_seen:
+          continue
+        # Increment the hit counter, since we got features out of this hit.
+        already_seen.add(already_seen_key)
+        for k in template_features:
+          template_features[k].append(result.features[k])
+
+    if already_seen:
+      for name in template_features:
+        template_features[name] = np.stack(
+            template_features[name], axis=0).astype(TEMPLATE_FEATURES[name])
+    else:
+      num_res = len(query_sequence)
+      # Construct a default template with all zeros.
+      template_features = {
+          'template_aatype': np.zeros(
+              (1, num_res, len(residue_constants.restypes_with_x_and_gap)),
+              np.float32),
+          'template_all_atom_masks': np.zeros(
+              (1, num_res, residue_constants.atom_type_num), np.float32),
+          'template_all_atom_positions': np.zeros(
+              (1, num_res, residue_constants.atom_type_num, 3), np.float32),
+          'template_domain_names': np.array([''.encode()], dtype=np.object),
+          'template_sequence': np.array([''.encode()], dtype=np.object),
+          'template_sum_probs': np.array([0], dtype=np.float32)
+      }
+    return TemplateSearchResult(
+        features=template_features, errors=errors, warnings=warnings)
