@@ -17,7 +17,7 @@
 The structure generation code is in 'folding.py'.
 """
 import functools
-from alphafold.common import residue_constants
+from alphafold.common import residue_constants, confidence_jax
 from alphafold.model import all_atom
 from alphafold.model import common_modules
 from alphafold.model import folding
@@ -370,20 +370,31 @@ class AlphaFold(hk.Module):
         return jnp.sqrt(jnp.abs(a_norm[:,None] + a_norm[None,:] - 2 * a @ a.T))
 
       def body(x):
-        n, tol, prev = x
-        prev_ = get_prev(do_call(prev, recycle_idx=n, compute_loss=False))
-        ca,ca_ = prev["prev_pos"][:,1,:], prev_["prev_pos"][:,1,:]
+        n, tol, _, prev = x
+        prediction = do_call(prev, recycle_idx=n, compute_loss=False)
+        confidences = confidence_jax.get_confidence_metrics(prediction, multimer_mode=self.config.global_config.multimer_mode)
+        # Get score for early stopping
+        if self.config.stop_at_score_ranker == "plddt":
+          # Masked mean, jax compatible
+          mean_score = (confidences["plddt"] * batch["seq_mask"]).sum() / batch["seq_mask"].sum()
+        else:
+          mean_score = confidences["ptm"].mean()
+        prev_ = get_prev(prediction)
+        ca, ca_ = prev["prev_pos"][:,1,:], prev_["prev_pos"][:,1,:]
         tol_ = jnp.sqrt(jnp.square(pw_dist(ca) - pw_dist(ca_)).mean())
-        return n+1, tol_, prev_
+        return n+1, tol_, mean_score, prev_
 
       if hk.running_init():
         # When initializing the Haiku module, run one iteration of the
         # while_loop to initialize the Haiku modules used in `body`.
-        recycles, tol, prev = body((0, jnp.inf, prev))
+        recycles, tol, _, prev = body((0, jnp.inf, 0, prev))
       else:
-        recycles, tol, prev = hk.while_loop(
-          lambda x: ((x[0] < num_iter) & (x[1] > self.config.recycle_tol)),
-          body,(0, jnp.inf, prev))
+        recycles, tol, _, prev = hk.while_loop(
+          # Stop when we reached the given number of recycles,
+          # or had less than a certain change since the last check (optional settings),
+          # or the chosen score is above the given threshold (optional setting)
+          lambda x: ((x[0] < num_iter) & (x[1] > self.config.recycle_tol)) & (x[2] < self.config.stop_at_score),
+          body,(0, jnp.inf, 0, prev))
     else:
       prev = {}
       num_iter = 0
@@ -973,6 +984,11 @@ class MaskedMsaHead(hk.Module):
     self.config = config
     self.global_config = global_config
 
+    if global_config.multimer_mode:
+      self.num_output = len(residue_constants.restypes_with_x_and_gap)
+    else:
+      self.num_output = config.num_output
+
   def __call__(self, representations, batch, is_training):
     """Builds MaskedMsaHead module.
 
@@ -989,7 +1005,7 @@ class MaskedMsaHead(hk.Module):
     """
     del batch
     logits = common_modules.Linear(
-        self.config.num_output,
+        self.num_output,
         initializer=utils.final_init(self.global_config),
         name='logits')(
             representations['msa'])
@@ -997,7 +1013,7 @@ class MaskedMsaHead(hk.Module):
 
   def loss(self, value, batch):
     errors = softmax_cross_entropy(
-        labels=jax.nn.one_hot(batch['true_msa'], num_classes=23),
+        labels=jax.nn.one_hot(batch['true_msa'], num_classes=self.num_output),
         logits=value['logits'])
     loss = (jnp.sum(errors * batch['bert_mask'], axis=(-2, -1)) /
             (1e-8 + jnp.sum(batch['bert_mask'], axis=(-2, -1))))
@@ -1017,7 +1033,7 @@ class PredictedLDDTHead(hk.Module):
     self.global_config = global_config
 
   def __call__(self, representations, batch, is_training):
-    """Builds ExperimentallyResolvedHead module.
+    """Builds PredictedLDDTHead module.
 
     Arguments:
       representations: Dictionary of representations, must contain:
@@ -1079,7 +1095,7 @@ class PredictedLDDTHead(hk.Module):
         # Shape (batch_size, num_res, 1)
         true_points_mask=all_atom_mask[None, :, 1:2].astype(jnp.float32),
         cutoff=15.,
-        per_residue=True)[0]
+        per_residue=True)
     lddt_ca = jax.lax.stop_gradient(lddt_ca)
 
     num_bins = self.config.num_bins
@@ -1605,6 +1621,19 @@ class EvoformerIteration(hk.Module):
     safe_key, *sub_keys = safe_key.split(10)
     sub_keys = iter(sub_keys)
 
+    outer_module = OuterProductMean(
+        config=c.outer_product_mean,
+        global_config=self.global_config,
+        num_output_channel=int(pair_act.shape[-1]),
+        name='outer_product_mean')
+    if c.outer_product_mean.first:
+      pair_act = dropout_wrapper_fn(
+          outer_module,
+          msa_act,
+          msa_mask,
+          safe_key=next(sub_keys),
+          output_act=pair_act)
+
     msa_act = dropout_wrapper_fn(
         MSARowAttentionWithPairBias(
             c.msa_row_attention_with_pair_bias, gc,
@@ -1632,16 +1661,13 @@ class EvoformerIteration(hk.Module):
         msa_mask,
         safe_key=next(sub_keys))
 
-    pair_act = dropout_wrapper_fn(
-        OuterProductMean(
-            config=c.outer_product_mean,
-            global_config=self.global_config,
-            num_output_channel=int(pair_act.shape[-1]),
-            name='outer_product_mean'),
-        msa_act,
-        msa_mask,
-        safe_key=next(sub_keys),
-        output_act=pair_act)
+    if not c.outer_product_mean.first:
+      pair_act = dropout_wrapper_fn(
+          outer_module,
+          msa_act,
+          msa_mask,
+          safe_key=next(sub_keys),
+          output_act=pair_act)
 
     pair_act = dropout_wrapper_fn(
         TriangleMultiplication(c.triangle_multiplication_outgoing, gc,
@@ -1738,8 +1764,7 @@ class EmbeddingsAndEvoformer(hk.Module):
                                           True,
                                           name='prev_msa_first_row_norm')(
                                               batch['prev_msa_first_row'])
-        msa_activations = jax.ops.index_add(msa_activations, 0,
-                                            prev_msa_first_row)
+        msa_activations = msa_activations.at[0].add(prev_msa_first_row)
 
       if 'prev_pair' in batch:
         pair_activations += hk.LayerNorm([-1],
